/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef CONSTS_H
#define CONSTS_H


//define some directories - may depend on Linux distribution
#define _DOCDIR "/usr/share/doc/"
#define _DATADIR "/usr/share/"
#define _BINDIR "/usr/bin/" 


/** Class holds all enums values for calendar application 
@brief Application enums class
@author Adam Skowroski
*/

class CalEnums 
{
public:
/**
Indicators of errors in XML file of periodic events 
*/
	enum XML_ERROR {
	ERR_DSTART	=1, 	/**< error in start date tag*/	
	ERR_DEND	=2, 	/**< error in end date tag*/ 	
	ERR_WEEKDAY	=4, 	/**< error in weekday tag */		
	ERR_PDAY	=8, 	/**< error in day_period tag */	
	ERR_PMONTH	=16, 	/**< error in month_period tag*/	
	ERR_PYEAR	=32, 	/**< error in year_period tag*/	
	ERR_PWEEK	=64, 	/**< error in week_period tag*/	
	ERR_GENERAL=128		/**< other error */				
	};

	
	/** Type of periodic event */
	enum event_type {
	 DAILY, 	/**< event is repeated every day */
	 WEEKLY, 	/**< event is repeated every week or every \em n weeks */
	 MONTHLY, 	/**< event is repeated every month or every \em n months */
	 YEARLY, 	/**< event is repeated every year or every \em n years */
	 PERIOD, 	/**< event is repeated every \em n days */
	 DAYS		/**< event is repeated in specified days in each week (ie. every Tuesday and Thurdsday) */
	 };
	
	
	/** States of calendar card. It indicates type of bitmap too.<br>
	The following list is in order from weakest to strongest. <br>
	It means: there is WEEKLY event on a card and YEARLY event is added. The card will be in PXY state */
	enum PIXMAP_STATES {
		NO_PIXMAP,	/**< there isn't any event */
		PXD, 		/**< there is CalEnums::DAILY or CalEnums::PERIOD event */
		PXW, 		/**< there is CalEnums::WEEKLY event */
		PXM, 		/**< there is CalEnums::MONTHLY event*/
		PXY, 		/**< there is CalEnums::YEARLY event*/
		PXI, 		/**< there is individual event */
		PXALERT	/**< there is an alert - currently used if event is in the current day */
		};
};
	
#endif
