/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "eperiod.h"

#include "iostream.h"

/*!
    \fn CalendarEventPeriodic::CalendarEventPeriodic()
	Default constructor of periodic event
 */
CalendarEventPeriodic::CalendarEventPeriodic()
 : CalendarEvent()
{
}


/*!
    \fn CalendarEventPeriodic::~CalendarEventPeriodic()
	Default destructor of periodic event
 */
CalendarEventPeriodic::~CalendarEventPeriodic()
{
}




/*!
    \fn CalendarEventPeriodic::CalendarEventPeriodic(QString text, QDate dStart, QDate dEnd, CalEnums::event_type eType, int period=1, bool *days=NULL, bool autodelete=TRUE, int forgotafter=0)

	Constructor of periodic event. It sets all data.
 	\param text event text
	\param dStart start of range date for periodical event
	\param dEnd end of range date for periodical event
	\param eType type of event (as in CalEnums::event_type)
	\param period period of periodical event
	\param days table with indicators for weekdays
	\param autodelete indicates if event will be automaticaly deleted after \c dEnd
	\param forgotafter indicates how long an event will be displayed in past calendar
*/
CalendarEventPeriodic::CalendarEventPeriodic(QString text, QDate dStart, QDate dEnd,  CalEnums::event_type eType, int period, bool *days, bool autodelete, int forgotafter) : CalendarEvent(dStart, text)
{
	this->eventType=eType;
	this->period=period;
	this->endDate=dEnd;
	if (days)
		{
		for (int i=0; i<7; i++) 
			this->days[i]=*(days+i);
		}
	this->maxDaysPassedForPeriodic=10;
	this->deleteFromFileAfterEnd=TRUE;
	this->deleteFromFileAfterEnd=autodelete;
	this->maxDaysPassedForPeriodic=forgotafter;
}


/*!
    \fn CalendarEventPeriodic::doesMatch(QDate d)
	Returns TRUE if a given date matches a periodic event, what means:
	 - date is between \c startDate and \c endDate
	 - date matches other conditions of event.
	\param d date to comparision
	\return \c TRUE if date is in range and date is in event day
 */
bool CalendarEventPeriodic::doesMatch(QDate d)
{
	if(d < startDate || d > endDate)
		return FALSE;
	switch (eventType)
	{
	case CalEnums::DAILY:
		return TRUE;
		break;
	case CalEnums::MONTHLY:
		if(d.day() == startDate.day() && !((d.month()-startDate.month()+(d.year()-startDate.year())*12)%period))
			return TRUE;
		else
			return FALSE;
		break;
	case CalEnums::WEEKLY:
		if(d.dayOfWeek() == startDate.dayOfWeek() && !(startDate.daysTo(d)%(period*7)))
			return TRUE;
		else
			return FALSE;
		break;
	case CalEnums::YEARLY:
		if(d.day() == startDate.day() && d.month() == startDate.month() && !((d.year()-startDate.year())%period)) 
			return TRUE;
		else
			return FALSE;
		break;
	case CalEnums::DAYS:
			return(days[d.dayOfWeek()-1]);
		break;
	case CalEnums::PERIOD:
		if(startDate.daysTo(d)%period)
			return FALSE;
		else
		    return TRUE;
		break;
	}
	return FALSE;
}


