/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qstring.h>
#include <qtextstream.h>
#include <qobject.h>
#include <iostream.h>
#include <qfile.h>

#include "file.h"
#include "findiv.h"
#include "fperiod.h"
#include "consts.h"
#include "fpast.h"

extern CalendarSettings config;

/*!
    \fn CalendarFile::CalendarFile()
	Default constructor.
 */
CalendarFile::CalendarFile()
{
}


/*!
    \fn CalendarFile::~CalendarFile()
	Default and empty destructor
 */
CalendarFile::~CalendarFile()
{
}


/*!
    \fn CalendarFile::saveFile(QString filename, QString text)
	Saves file. Text is written to filename file
	\param filename filename of file to save
	\param text text to save in file.
 */
void CalendarFile::saveFile(QString filename, QString text)
{
	QString error;
    
	QFile file( filename );
    if ( !file.open( IO_WriteOnly ) )
		{
		cout << QObject::tr("File ") << filename << QObject::tr(" cannot be opened\nCheck if the file exists and if you have a sufficient rights\n");
		}
	else
		{
		QTextStream stream( &file );
        stream << text;
        file.close();
		}
}


/*!
    \fn CalendarFile::savePeriodicEvents(QPtrList<CalendarEventPeriodic> * list)
	Saves list of periodic events into xml file.
 	\param list pointer to application list of periodic events
*/
void CalendarFile::savePeriodicEvents(QPtrList<CalendarEventPeriodic> * list)
{
	FilePeriodic fp(list);
	QString tekst= fp.createPeriodicXML();
	this->saveFile(::config.PATH+periodic_file_name, tekst);
}


/*!
    \fn CalendarFile::saveIndividualEvents(QPtrList<CalendarEvent> * list)
	Saves list of individual events into xml file.
 	\param list pointer to application list of individual events
 */
void CalendarFile::saveIndividualEvents(QPtrList<CalendarEvent> * list)
{
	FileIndividual fi(list);
	QString tekst = fi.createIndividualXML();
	this->saveFile(::config.PATH+individual_file_name, tekst);
}



/*!
    \fn CalendarFile::loadIndividualEvents(QPtrList<CalendarEvent> * list)
	Method loads individual events from XML file into list of individual events.
 	\param list pointer to application list of individual events
 */
void CalendarFile::loadIndividualEvents(QPtrList<CalendarEvent> * list)
{
	FileIndividual fi(list);
	fi.createIndividualList(::config.PATH+individual_file_name);
}


/*!
    \fn CalendarFile::loadPeriodicEvents(QPtrList<CalendarEventPeriodic> * list)
	Method loads periodic events from XML file into list of periodic events.
 	\param list pointer to application list of periodic events
 */
void CalendarFile::loadPeriodicEvents(QPtrList<CalendarEventPeriodic> * list)
{
    FilePeriodic fp(list);
	fp.createPeriodicList(::config.PATH+periodic_file_name);
}




/*!
    \fn CalendarFile::savePastEvents(QPtrList<CalendarEventPast> * list)
	Method saves list of past events into past events file
	\param list pointer to application list of past events
 */
void CalendarFile::savePastEvents(QPtrList<CalendarEventPast> * list)
{
	FilePast fp(list);
	QString tekst = fp.createPastXML();
	this->saveFile(::config.PATH+past_file_name, tekst);
}


/*!
    \fn CalendarFile::loadPastEvents(QPtrList< CalendarEventPast > * list)
	Method load past events from filename
	\param list pointer to application list of past events where events will be loaded
 */
QDate CalendarFile::loadPastEvents(QPtrList< CalendarEventPast > * list)
{
    FilePast fp(list);
	return fp.createPastList(::config.PATH+past_file_name);
}
