/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "fxml.h"
#include <qstring.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <iostream.h>

/*!
	\fn FileXML::FileXML()
	Default constructor
*/
FileXML::FileXML()
{
}


/*!
	\fn FileXML::~FileXML()
	Default destructor
*/
FileXML::~FileXML()
{
}
/*!
    \fn FileXML::addTagAny(QString tag, int level, QString value)
	Method creates string for one data xml tag
 	\param tag name of a tag
	\param level number of tabulation at the beginning of line
	\param value value of a tag
	\return XML tag string
*/
QString FileXML::addTagAny(QString tag, int level, QString value)
{
	QString temp;
	while (level--)
			temp+="\t";
    if(value=="")
		return temp+="<"+tag+"/>\n";
	else
		return temp+="<"+tag+">"+value+"</"+tag+">\n";
}




/*!
    \fn FileXML::openXMLFileForRead(QString filename, QString root, QDomElement* elem)
 	\param filename name of file to be opened
	\param root name of root tag of xml file
	\param elem pointer to xml element. This will be returned to calling function
	\return \c 0 for success \n \c 1 for failure
	\n \em elem as a pointer - handle of XML structure been read from file
 */
int FileXML::openXMLFileForRead(QString filename, QString root, QDomElement* elem)
{
	QString error;
	int x,y;
    QDomDocument doc( root );
    QFile file( filename );
    if ( !file.open( IO_ReadOnly ) )
		{
		cout << QObject::tr("Cant open file ") << filename << QObject::tr("\nCheck if the file exists and if you have sufficient rights to read it. If the file doesn't exist, the new, blank one, will be created automaticaly.\n\n");
        return 1;
		}
    if ( !doc.setContent( &file, &error, &y, &x ) ) {
		QString temp=QObject::tr("Cant read data from XML file\nError description:\nline ");
		temp+=QString("%1").arg(y);
		temp+=QObject::tr(", character ");
		temp+=QString("%1").arg(x);
		temp+=": ";
		temp+=error;
		temp+="\n";
        file.close();
		QMessageBox a(QObject::tr("Error in XML file"), temp, QMessageBox::Warning,0,0,0);
		a.exec();
        return 2;
		}
    file.close();
    
	*elem = doc.documentElement();
	return 0;
}

