/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qstring.h>
#include <stdio.h>

#include "wcalendar.h"

extern CalendarSettings config;



/*!
    \fn CalendarWidget::~CalendarWidget()
	Default destructor of class
 */
CalendarWidget::~CalendarWidget()
{

}


/*!
    \fn CalendarWidget::closeInfos()
	Method to close InfoWidget windows dependently on closeOtherInfoWindows property.
 */
void CalendarWidget::closeInfos()
{
	//jesli tak jest w konfiguracji

	if (::config.closeOtherInfoWindows)
		{
		for (uint i=0; i<kartki.count(); i++)
			{
			if (!kartki.at(i)->isInfoNull())
				{
				kartki.at(i)->setOn(FALSE);
				//return;
				}
			}
		}
	return;
}









/*!
    \fn CalendarWidget::resetCalendar()
	Reset settings of card widgets. Used when pixmap corresponding to events are changing
 */
void CalendarWidget::resetCalendar()
{
	uint i;
	CalendarCardWidget* guzik;
	
	for (i=0;i<kartki.count();i++)
	{
		guzik = kartki.at(i);
		guzik->initialiseCardWidgets();
		guzik->repaint();
	}
}

/*!
    \fn CalendarWidget::changeGeometry(int pos, int width)
	changes geometry of calendar (position and width)
	\param pos relative position of left side of calendar inside application window
	\param width width of calendar window
 */
void CalendarWidget::changeGeometry(int pos, int width)
{
   	this->pos=pos;
	this->width=width-pos;
	setGeometry(pos, 0, width, ::config.HEIGHT+::config.MONTH_HEIGHT);
}






/*!
    \fn CalendarWidget::CalendarWidget(QWidget *parent, const char *name, int f)
	Constructor of application window. 
		- Sets properties to initial values, 
		- creates GUI objects
		- creates CalendarCardData objects
		- create CalendarCardWidget objects
		- loads IndividualEvents.xml
		- loads PeriodicEvents.xml 
	\param parent pointer to parent widget
	\param name name of widget
	\param f I don't know!
 */
CalendarWidget::CalendarWidget(QWidget *parent, const char *name, int f) 
: QWidget(parent, name, f)
{
    this->parent=(Calendar*)parent;
//preparation of lists of events
	kartki.setAutoDelete( TRUE );
//    oldEvents.setAutoDelete( TRUE );
	dane.setAutoDelete( TRUE );
	
}

/** \name Initialisers */
/* @{ */



/*!
    \fn CalendarWidget::createCalendarCards(QPtrList<CalendarCardWidget>* lista, int ile)
	Initialisation method
	<br>Creates CalendarCardWidget objects.
	\param lista global list of CalendarCardWidget objects
	\param ile number of widgets to be added to list
 */
void CalendarWidget::createCalendarCards(QPtrList<CalendarCardWidget>* lista, int ile)
{
	int i;
	for (i=0;i<ile;i++)
	{
		CalendarCardWidget* guzik;
		guzik = new CalendarCardWidget(this, "Day");
		guzik->setGeometry( QRect( 0, 0, ::config.WIDTH, ::config.HEIGHT ) );
		guzik->initialiseCardWidgets();
		guzik->setAutoMask( TRUE );
		guzik->setPixmap( *(getPixmap(0)) );
		guzik->setToggleButton( TRUE );
		clearWState( WState_Polished );
		
		lista->append(guzik);
	}
	resize( QSize(::config.kalendarzWidth, ::config.HEIGHT+::config.MONTH_HEIGHT));
}
/*@}*/




/*!
    \fn CalendarWidget::getPixmap(int index)
	\param index index of type of event
	\return Pixmap assigned to this type of event
 */
QPixmap* CalendarWidget::getPixmap(int index)
{
	return ::config.getPixmap(index);
}





/*!
    \fn CalendarWidget::removeCardData(QPtrList<CalendarCardData> * lista, long from)
	Method used to remove unused CalendarCardData objects from a list. All CalendarCardData which index is greater then current index + 365 are removed from memory.
	\param lista pointer to global list of CalendarCardData objects
	\param from indext of last object left in the list
*/
void CalendarWidget::removeCardData(QPtrList<CalendarCardData> * lista, long from)
{
	for (int i=lista->count(); i>from; i--)
		lista->remove(i);
}
