/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "wfuture.h"
#include "iostream.h"
#include <qfont.h>
#include <qpainter.h>

extern CalendarSettings config;

/**
	\fn CalendarWidgetFuture::CalendarWidgetFuture(QWidget *parent, const char *name, int f)

	Constructor of a class. 
	\param parent parent widget (application window)
	\param name unused 
	\param f I don't know
*/
CalendarWidgetFuture::CalendarWidgetFuture(QWidget* parent, const char* name, int f): CalendarWidget(parent, name, f)
{
	//creation of buttons
	createCalendarCards(&kartki, ::config.noOfKalWidgets);
	
	//creation of data 
	createCalendarData(&dane, ::config.noOfKalWidgets);
	
	zero_position = 0; //this could change in a future

	
	//setting visible buttons as active
	associateCardsDatas(&kartki, &dane);
}


CalendarWidgetFuture::~CalendarWidgetFuture()
{
}


/*!
    \fn CalendarWidgetFuture::shift(int i)
	Moves all CalendarCardWidget windows by i pixels to right of by -i to left
	\param i value of shift.
 */
void CalendarWidgetFuture::shift(int i)
{
	int j;
	CalendarCardWidget *temp=NULL;
	
	if (zero_position+i>0)
		if (zero_position==0)
			return;
		else
			{
			i=-zero_position;
			zero_position=0;
			}
	else
		zero_position+=i;
	
	if (i<0) //przesuwamy w lewo
	{
		for (j=0; j<(int)::config.noOfKalWidgets; j++)
			{
				temp=kartki.at(j);
				if (temp->position+i < -::config.WIDTH )
					{
					//przerzucamy to na prawa strone
					if(temp->numControl+::config.noOfKalWidgets >= (int)dane.count())
						this->addCardData(&dane, dane.count() -temp->numControl);
					temp->moveCalendarCard(i+::config.noOfKalWidgets*::config.WIDTH);
					temp->freeInfoWidget();
					temp->pControl->pWidget=NULL;
					temp->numControl+=::config.noOfKalWidgets;					temp->associateCardData(dane.at(temp->numControl));
					temp->updateState(temp->pControl->state);
					}
				else
					temp->moveCalendarCard(i);

			}
	}
	if (i>0) //przesuwamy w prawo
	{
		for (j=(int)::config.noOfKalWidgets-1; j>=0; j--)
		{
			temp=kartki.at(j);
			if (temp->position+i > ::config.screenWidth)
				{
				//przerzucamy to na lew stron.
				temp->moveCalendarCard(i-::config.noOfKalWidgets*::config.WIDTH);
				temp->freeInfoWidget();
				temp->pControl->pWidget=NULL;
				temp->numControl-=::config.noOfKalWidgets;
				if(temp->numControl>=0)
					{
					temp->associateCardData(dane.at(temp->numControl));
					temp->updateState(temp->pControl->state);
					}
				}
			else
			{
				temp->moveCalendarCard(i);
				if (temp->position+i > this->width)
					temp->freeInfoWidget();
			}
		}
		if(dane.count() > (uint)(temp->numControl)+370)
			removeCardData(&dane, temp->numControl+370);
	}
	if(abs(i)>::config.WIDTH)
		{
			for (uint a=0; (int)a < ::config.noOfKalWidgets; a++)
				kartki.at(a)->repaint();	
		}
	repaint(QRect(0,::config.HEIGHT+1, ::config.kalendarzWidth, ::config.HEIGHT+::config.MONTH_HEIGHT ));
}

/*!
    \fn CalendarWidgetFuture::keyPressEvent ( QKeyEvent * e )
	Overloaded member of QWidget::keyPressEvent method.
	Provides shifting of calendar using keyboard.
	\param e system event object
 */
void CalendarWidgetFuture::keyPressEvent ( QKeyEvent * e )
{
	QDate date;
	switch(e->key())
	{
		case Qt::Key_Left:	
				shift(-::config.WIDTH);
				break;
		case Qt::Key_Right:	
				shift(::config.WIDTH);
				break;
		case Qt::Key_Up:	
				shift(-7*::config.WIDTH);
				break;
		case Qt::Key_Down:	
				shift(7*::config.WIDTH);
				break;
		case Qt::Key_Prior:	
				//@TODO implement pgup
				cout << "PgUp\n";
				break;
		case Qt::Key_Next:	
				//@TODO implement PgDown
				cout << "PgDown\n";
				break;
		case Qt::Key_Home:
				date.currentDate();
				switchToDate(date);
				break;
		default: 
			break;
	}
}
/*!
    \fn CalendarWidgetFuture::createCalendarData(QPtrList<CalendarCardData>* lista, int ile)
	Initialisation method
	<br>Creates CalendarCardData objects.
	\param lista pointer to global list of CalendarCardData objects
	\param ile number of new CalendarCardData objects has to be added
 */
void CalendarWidgetFuture::createCalendarData(QPtrList<CalendarCardData>* lista, int ile)
{
	int i;
	for (i=0;i<ile;i++)
	{
		CalendarCardData* dzien;
		dzien = new CalendarCardData(QDate::currentDate().addDays(i), this);
		lista->append(dzien);
	}
	dispatchIndividualEvents(0, ile-1);
}


/*!
    \fn CalendarWidgetFuture::paintEvent( QPaintEvent * e)
	Overloaded method of QWidget class. 
		- draws CalendarCardWidget objects
		- draws dates on these objects
		- draws names of months.
	\param e system event object
 */
void CalendarWidgetFuture::paintEvent( QPaintEvent * e)
{
	QPainter paint(this);
	paint.setPen(::config.theme.monthFontColor);
	paint.setFont(::config.theme.monthFont);

	CalendarCardWidget *temp;
	int ile=this->kartki.count();
	int i;
	bool t=FALSE;
	
	for (i=0; i<ile; i++)
	{
		temp=this->kartki.at(i);
		if (!temp->bFree && temp->pControl && temp->pControl->isFirstOfMonth and temp->position>0)
			{
			t=TRUE;
			paint.drawLine(temp->pos().x(), temp->pos().y()+::config.HEIGHT, temp->pos().x(), temp->pos().y()+::config.HEIGHT+::config.MONTH_HEIGHT);
			if( temp->pControl->getDate().dayOfYear()==1)
			{
				paint.drawText(QRect(temp->pos().x()+1, temp->pos().y()+::config.HEIGHT, ::config.WIDTH*3, ::config.MONTH_HEIGHT), AlignLeft, QDate::longMonthName(temp->pControl->getDate().month())+", " + QString::number(temp->pControl->getDate().year()));
				}
			else
				{
				paint.drawText(QRect(temp->pos().x()+1, temp->pos().y()+::config.HEIGHT, ::config.WIDTH*2, ::config.MONTH_HEIGHT), AlignLeft, QDate::longMonthName(temp->pControl->getDate().month()));
				}
		}	
		else if(!temp->bFree && temp->pControl && temp->position>=0 && temp->position < ::config.WIDTH)
		{
			paint.drawText(QRect(1, temp->pos().y()+::config.HEIGHT, ::config.WIDTH*2, ::config.MONTH_HEIGHT), AlignLeft, QDate::longMonthName(temp->pControl->getDate().month()));
		}
	}
}

/*!
    \fn CalendarWidgetFuture::associateCardsDatas(QPtrList<CalendarCardWidget> * kartki, QPtrList<CalendarCardData> * daty)
	Associates CalendarCardData with CalendarCardWidget objects.
	<Br>Sets all necessary data for visible CalendarCardWidget objects.
	\param kartki pointer to list of CalendarCardWidgets object
	\param daty pointer to list of CalendarCardData object
 */
void CalendarWidgetFuture::associateCardsDatas(QPtrList<CalendarCardWidget> * kartki, QPtrList<CalendarCardData> * daty)
{
	for (uint i=0; i<kartki->count(); i++)
	{	
		kartki->at(i)->initialiseCardData(zero_position+i*::config.WIDTH, daty->at(i), i);
	}
}


/*!
    \fn CalendarWidgetFuture::addCardData(QPtrList<CalendarCardData>* lista, int ile)
	Creates \c ile of new CalendarCardData objects. 
	\param lista pointer to global list of CalendarCardData objects
	\param ile number of new CalendarCardData objects has to be added
 */
void CalendarWidgetFuture::addCardData(QPtrList<CalendarCardData>* lista, int ile)
{
	int delta_daty;
	int i;
	int ile_dotad=lista->count();

	for (i=0;i<ile;i++)
	{
		delta_daty=i+ile_dotad;
		CalendarCardData* dzien;
		dzien = new CalendarCardData(QDate::currentDate().addDays(delta_daty), this);
		lista->append(dzien);
	}
	dispatchIndividualEvents(ile_dotad, ile_dotad+ile-1);
	dispatchPeriodicEvents(ile_dotad, ile_dotad+ile-1);
}


/*!
    \fn CalendarWidgetFuture::switchToDate(QDate date)
	Method to display imidiately calendar with a specified date
	\param date specified date
 */
void CalendarWidgetFuture::switchToDate(QDate date)
{
	//sprawdzenie czy data < current date
	QDate current_date;
	uint daysTo;
	
	current_date.currentDate();
	
	if (date < current_date)
	 	date=current_date;
	
	daysTo=current_date.daysTo(date);
	
	zero_position=daysTo*::config.WIDTH;
	
	if (dane.count() <= daysTo)
		addCardData(&dane, daysTo-dane.count()+::config.noOfKalWidgets*2);
	
	for (uint i = 0; (int)i < ::config.noOfKalWidgets;i++)
		{
			this->kartki.at(i)->initialiseCardData(i*::config.WIDTH, dane.at(daysTo+i), daysTo+i);
		}

	if(dane.count() > daysTo+370)
		removeCardData(&dane, daysTo+370);

				
	repaint(QRect(0,::config.HEIGHT+1, ::config.kalendarzWidth, ::config.HEIGHT+::config.MONTH_HEIGHT ));	

}

/*!
    \fn CalendarWidgetFuture::removeIndividualEvent(CalendarEvent* event)
	Removes individual event from a global list of individual events
	\param event individual event has to be removed
 */
void CalendarWidgetFuture::removeIndividualEvent(CalendarEvent* event)
{
	allIndividualEvents.removeRef(event);
	parent->removeIndividualEvent(event);
	
}


/*!
    \fn CalendarWidgetFuture::addIndividualEvent(CalendarEvent *event)
	Adds individual event to a list of individual events
	\param event event object
 */
void CalendarWidgetFuture::addIndividualEvent(CalendarEvent *event)
{
	this->allIndividualEvents.append(event);
	
}

/*!
    \fn CalendarWidgetFuture::addIndividualEvent(QDate date, QString text)
	Adds individual event to a list of individual events
	\param date event date
	\param text event text
 */
CalendarEvent* CalendarWidgetFuture::addIndividualEvent(QDate date, QString text)
{
	return this->parent->addIndividualEvent(date,text);
}





/*!
    \fn CalendarWidgetFuture::addPeriodicEvent(CalendarEventPeriodic* event	)
	Method adds periodic events into calendar and cause that new events will be displayed immediately
	\param event periodic event object
 */
void CalendarWidgetFuture::addPeriodicEvent(CalendarEventPeriodic* event)
{
	this->allPeriodicEvents.append(event);
	
	this->dispatchPeriodicEvents(QDate::currentDate().daysTo(event->startDate), QDate::currentDate().daysTo(event->endDate));
	

}

/*!
	 \fn CalendarWidgetFuture::addPeriodicEvent(QDate dStart, QDate dEnd, QString t, CalEnums::event_type type, int period, bool* days)
	Method adds periodic events into calendar and cause that new events will be displayed immediately
	\param dStart start of range date
	\param dEnd end of range date
	\param t text of event
	\param type type of event
	\param period period of event
	\param days table with indicators for weekdays
	\return pointer to periodic event object
*/
CalendarEventPeriodic* CalendarWidgetFuture::addPeriodicEvent(QDate dStart, QDate dEnd, QString t, CalEnums::event_type type, int period, bool* days)
{
	return parent->addPeriodicEvent(dStart, dEnd, t, type, period, days);
}


/*!
    \fn CalendarWidgetFuture::removePeriodicEvent(CalendarEventPeriodic * event)
	Removes periodic event from a global list of periodic events
	\param event object which will be removed
 */
void CalendarWidgetFuture::removePeriodicEvent(CalendarEventPeriodic * event)
{
	unsigned int i=0;
	while (allPeriodicEvents.at(i)!=event && i<allPeriodicEvents.count())
		i++;
	if (i<allPeriodicEvents.count())
	{
		int start, end;
		start = QDate::currentDate().daysTo(allPeriodicEvents.at(i)->startDate);
		end = QDate::currentDate().daysTo(allPeriodicEvents.at(i)->endDate);
		//remove from Controls
		clearPeriodicEvents(start, end);
		//remove from list
		allPeriodicEvents.remove(i);
		//dispatch to Controls
		dispatchPeriodicEvents(start, end);
	}
}

/*!
    \fn CalendarWidgetFuture::clearPeriodicEvents(int start, int end)
	Removes all periodic events from CalendarCardData object with indexes from start to end
	\param start start of range of indexes
	\param end end of range of indexes
 */
void CalendarWidgetFuture::clearPeriodicEvents(int start, int end)
{
	CalendarCardData *temp;
	if (start >= (signed int)(this->dane.count()))
		return;
	
	if(end >= (signed int)(this->dane.count()))
		end=dane.count()-1;
	if(start < 0)
		start = 0;
	
	
	for (int i=start; i<=end; i++)
		{
		temp=this->dane.at(i);
		if(temp->pWidget)
			temp->pWidget->updateState(temp->removePeriodicEvents());
		else 
			temp->removePeriodicEvents();
		}
}


/*!
    \fn CalendarWidgetFuture::getPeriodicEvent(int i)
	Returns periodic event object
	\param i index on the global list of periodical events
	\return Periodical event object
 */
CalendarEventPeriodic* CalendarWidgetFuture::getPeriodicEvent(int i)
{
   if(i<(signed long)(this->allPeriodicEvents.count()))
		return allPeriodicEvents.at(i);
	else 
		return NULL;
}



/*!
    \fn CalendarWidgetFuture::loadIndividualEvents(QPtrList<CalendarEvent>* allI)
	Loads individual events from a file and displays new events on the screen. 
	\param allI list of individual events
	\warning verify description
 */
void CalendarWidgetFuture::loadIndividualEvents(QPtrList<CalendarEvent>* allI)
{
	QDate cur=QDate::currentDate();
	for (uint i=0; i< allI->count();i++)
		{
			if(allI->at(i)->startDate>=cur)
				this->allIndividualEvents.append(allI->at(i));
		}
	
	this->dispatchIndividualEvents(0, this->dane.count()-1);
}


/*!
    \fn CalendarWidgetFuture::loadPeriodicEvents(QPtrList<CalendarEventPeriodic>* allP)
 	Loads periodic events from a file and displays new events on the screen
	\param allP list of periodic events
 */
void CalendarWidgetFuture::loadPeriodicEvents(QPtrList<CalendarEventPeriodic>* allP)
{
	QDate cur=QDate::currentDate();
	for (uint i=0; i< allP->count();i++)
		{
			if(allP->at(i)->endDate>=cur)
				this->allPeriodicEvents.append(allP->at(i));
		}
	
	this->dispatchPeriodicEvents(0, this->dane.count()-1);
}


/*!
    \fn CalendarWidgetFuture::dispatchPeriodicEvents(int start, int end)
	Adds periodical events to CalendarCardData objects for object from start to end indexes
	\param start start of range of indexes
	\param end end of range of indexes
 */
void CalendarWidgetFuture::dispatchPeriodicEvents(int start, int end)
{
	CalendarEventPeriodic* temp;
	if(start >= (signed long)(dane.count()))
		return;
	if(end >= (signed long)(dane.count()))
		end = dane.count()-1;
	if(start < 0)
		start = 0;
		
	this->clearPeriodicEvents(start, end);
	for(uint j=0; j<allPeriodicEvents.count();j++)
	{
		temp=allPeriodicEvents.at(j);
		for (int i = start; i<=end; i++)
		{
			if(temp->doesMatch(dane.at(i)->getDate()))
			{
				dane.at(i)->addPeriodicEvent(temp);
			}
		}
	}
}




/*!
    \fn CalendarWidgetFuture::dispatchIndividualEvents(int start, int end)
	Adds individual events to CalendarCardData objects for object from start to end indexes
	\param start start of range of indexes
	\param end end of range of indexes
 */
void CalendarWidgetFuture::dispatchIndividualEvents(int start, int end)
{
	CalendarEvent* temp;
	CalendarCardData* tempControl;
	for(uint j=0; j<allIndividualEvents.count();j++)
	{
		temp=allIndividualEvents.at(j);
		for (int i = start; i<=end; i++)
		{
			tempControl=dane.at(i);
			if(temp->doesMatch(tempControl->getDate()))
			{
				tempControl->addIndividualEvent(temp);
				if(tempControl->pWidget)
				{
					tempControl->pWidget->updateState(CalEnums::NO_PIXMAP);
					tempControl->pWidget->updateState(tempControl->state);
				}

			}
		}
	}
}


